@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.5.2.3 2007/08/01 04:26:57 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.5.2.5 2007/09/14 11:06:35 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -66,7 +66,6 @@ static RETSIGTYPE close_idle_connection(int sig);
 static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 static void send_frontend_exits(void);
 static int s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password);
-static int select_load_balancing_node(void);
 
 /*
  * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
@@ -1056,7 +1055,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 	if (backend == NULL)
 	{
 		pool_send_error_message(frontend, sp->major, "XX000", "connection cache is full", "",
-								"increace max_pool", __FILE__, __LINE__);
+								"increase max_pool", __FILE__, __LINE__);
 		pool_close(frontend);
 		pool_free_startup_packet(sp);
 		return NULL;
@@ -1646,7 +1645,7 @@ static int s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password)
 /*
  * Select load balancing node
  */
-static int select_load_balancing_node(void)
+int select_load_balancing_node(void)
 {
 	double total_weight,r;
 	int i;
