@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.5 2007/05/17 06:46:02 yamaguti Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.6.2.2 2007/07/09 05:00:38 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -22,11 +22,15 @@
 
 #include <ctype.h>
 #include <sys/types.h>
+#include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <sys/un.h>
 #include <netdb.h>
 #include <arpa/inet.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -47,6 +51,20 @@
 
 #include "version.h"
 
+#define CHECK_REQUEST \
+	do { \
+		if (failover_request) \
+		{ \
+			failover(); \
+			failover_request = 0; \
+		} \
+		if (sigchld_request) \
+		{ \
+			reaper(); \
+		} \
+    } while (0)
+
+
 #define PGPOOLMAXLITSENQUEUELENGTH 10000
 static void daemonize(void);
 static int read_pid_file(void);
@@ -56,6 +74,10 @@ static pid_t fork_a_child(int unix_fd, int inet_fd, int id);
 static int create_unix_domain_socket(struct sockaddr_un un_addr_tmp);
 static int create_inet_domain_socket(const char *hostname, const int port);
 static void myexit(int code);
+static void failover(void);
+static void reaper(void);
+static int pool_pause(struct timeval *timeout);
+static void pool_sleep(unsigned int second);
 
 static RETSIGTYPE exit_handler(int sig);
 static RETSIGTYPE reap_handler(int sig);
@@ -98,6 +120,9 @@ static int stop_sig = SIGTERM;	/* stopping signal default value */
 static int health_check_timer_expired;		/* non 0 if health check timer expired */
 
 POOL_REQUEST_INFO *Req_info;		/* request info area in shared memory */
+static volatile sig_atomic_t failover_request = 0;
+static volatile sig_atomic_t sigchld_request = 0;
+static int pipe_fds[2]; /* for delivering signals */
 
 int my_proc_id;
 
@@ -340,6 +365,7 @@ int main(int argc, char **argv)
 	}
 
 	/* set up signal handlers */
+	POOL_SETMASK(&BlockSig);
 	pool_signal(SIGTERM, exit_handler);
 	pool_signal(SIGINT, exit_handler);
 	pool_signal(SIGQUIT, exit_handler);
@@ -347,6 +373,13 @@ int main(int argc, char **argv)
 	pool_signal(SIGUSR1, failover_handler);
 	pool_signal(SIGHUP, exit_handler);
 
+	/* create pipe for delivering event */
+	if (pipe(pipe_fds) < 0)
+	{
+		pool_error("failed to create pipe");
+		myexit(1);
+	}
+
 	pool_log("pgpool successfully started");
 
 	/* fork a child for PCP handling */
@@ -366,6 +399,8 @@ int main(int argc, char **argv)
 	 */
 	for (;;)
 	{
+		CHECK_REQUEST;
+
 		/* do we need health checking for PostgreSQL? */
 		if (pool_config->health_check_period > 0)
 		{
@@ -413,7 +448,7 @@ int main(int argc, char **argv)
 						pool_log("set %d th backend down status", sts);
 						Req_info->kind = NODE_DOWN_REQUEST;
 						Req_info->node_id = sts;
-						failover_handler(SIGUSR1);
+						failover();
 						/* need to distribute this info to children */
 					}
 					else
@@ -427,7 +462,7 @@ int main(int argc, char **argv)
 							pool_log("set %d th backend down status", sts);
 							Req_info->kind = NODE_DOWN_REQUEST;
 							Req_info->node_id = sts;
-							failover_handler(SIGUSR1);
+							failover();
 							retrycnt = 0;
 						}
 						else
@@ -435,10 +470,7 @@ int main(int argc, char **argv)
 							/* continue to retry */
 							sleep_time = pool_config->health_check_period/NUM_BACKENDS;
 							pool_debug("retry sleep time: %d seconds", sleep_time);
-							while (sleep_time > 0)
-							{
-								sleep_time = sleep(sleep_time);
-							}
+							pool_sleep(sleep_time);
 							continue;
 						}
 					}
@@ -458,8 +490,7 @@ int main(int argc, char **argv)
 					{
 						sleep_time = pool_config->health_check_period/NUM_BACKENDS;
 						pool_debug("retry sleep time: %d seconds", sleep_time);
-						while (sleep_time > 0)
-							sleep_time = sleep(sleep_time);
+						pool_sleep(sleep_time);
 						continue;
 					}
 				}
@@ -472,14 +503,20 @@ int main(int argc, char **argv)
 			}
 
 			sleep_time = pool_config->health_check_period;
-			while (sleep_time > 0)
-			{
-				sleep_time = sleep(sleep_time);
-			}
+			pool_sleep(sleep_time);
 		}
 		else
 		{
-			pause();
+			for (;;)
+			{
+				int r;
+
+				POOL_SETMASK(&UnBlockSig);
+				r = pool_pause(NULL);
+				POOL_SETMASK(&BlockSig);
+				if (r > 0)
+					break;
+			}
 		}
 	}
 	pool_shmem_exit(0);
@@ -641,6 +678,9 @@ pid_t pcp_fork_a_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 
 	if (pid == 0)
 	{
+		close(pipe_fds[0]);
+		close(pipe_fds[1]);
+
 		myargv = save_ps_display_args(myargc, myargv);
 
 		/* call PCP child main */
@@ -666,6 +706,9 @@ pid_t fork_a_child(int unix_fd, int inet_fd, int id)
 
 	if (pid == 0)
 	{
+		close(pipe_fds[0]);
+		close(pipe_fds[1]);
+
 		myargv = save_ps_display_args(myargc, myargv);
 
 		/* call child main */
@@ -929,17 +972,27 @@ static RETSIGTYPE exit_handler(int sig)
 
 
 /*
- * handle SIGUSR1 (backend connection error, failover/failback request, if possible)
+ * handle SIGUSR1
  *
  */
 static RETSIGTYPE failover_handler(int sig)
+{
+	POOL_SETMASK(&BlockSig);
+	failover_request = 1;
+	write(pipe_fds[1], "\0", 1);
+	POOL_SETMASK(&UnBlockSig);
+}
+
+/*
+ * backend connection error, failover/failback request, if possible
+ * failover() must be called under protecting signals.
+ */
+static void failover(void)
 {
 
 	int node_id;
 	int i;
 
-	POOL_SETMASK(&BlockSig);
-
 	pool_debug("failover_handler called");
 
 	/*
@@ -949,7 +1002,6 @@ static RETSIGTYPE failover_handler(int sig)
 	if (getpid() != mypid)
 	{
 		pool_debug("failover_handler: I am not parent");
-		POOL_SETMASK(&UnBlockSig);
 		return;
 	}
 
@@ -959,7 +1011,6 @@ static RETSIGTYPE failover_handler(int sig)
 	if (exiting)
 	{
 		pool_debug("failover_handler called while exiting");
-		POOL_SETMASK(&UnBlockSig);
 		return;
 	}
 
@@ -969,7 +1020,6 @@ static RETSIGTYPE failover_handler(int sig)
 	if (switching)
 	{
 		pool_debug("failover_handler called while switching");
-		POOL_SETMASK(&UnBlockSig);
 		return;
 	}
 
@@ -978,7 +1028,6 @@ static RETSIGTYPE failover_handler(int sig)
 	if (node_id < 0)
 	{
 		pool_error("failover_handler: invalid node_id %d", node_id);
-		POOL_SETMASK(&UnBlockSig);
 		return;
 	}
 
@@ -988,7 +1037,6 @@ static RETSIGTYPE failover_handler(int sig)
 	{
 		pool_error("failover_handler: invalid node_id %d status:%d MAX_NUM_BACKENDS: %d", node_id,
 					BACKEND_INFO(node_id).backend_status, MAX_NUM_BACKENDS);
-		POOL_SETMASK(&UnBlockSig);
 		return;
 	}
 
@@ -1051,7 +1099,6 @@ static RETSIGTYPE failover_handler(int sig)
 			}
 
 			switching = 0;
-			POOL_SETMASK(&UnBlockSig);
 			return;
 		}
 
@@ -1101,7 +1148,6 @@ static RETSIGTYPE failover_handler(int sig)
 
 	Req_info->node_id = -1;
 	switching = 0;
-	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -1267,25 +1313,34 @@ system_db_health_check(void)
  * handle SIGCHLD
  */
 static RETSIGTYPE reap_handler(int sig)
+{
+	POOL_SETMASK(&BlockSig);
+	sigchld_request = 1;
+	write(pipe_fds[1], "\0", 1);
+	POOL_SETMASK(&UnBlockSig);
+}
+
+/*
+ * Attach zombie processes and restart child processes.
+ * reaper() must be called under protecting signals.
+ */
+static void reaper(void)
 {
 	pid_t pid;
 	int status;
 	int i;
 
-	POOL_SETMASK(&BlockSig);
-
 	pool_debug("reap_handler called");
+	sigchld_request = 0;
 
 	if (exiting)
 	{
-		POOL_SETMASK(&UnBlockSig);
 		pool_debug("reap_handler: exited due to exiting");
 		return;
 	}
 
 	if (switching)
 	{
-		POOL_SETMASK(&UnBlockSig);
 		pool_debug("reap_handler: exited due to swicting");
 		return;
 	}
@@ -1327,7 +1382,6 @@ static RETSIGTYPE reap_handler(int sig)
 		}
 	}
 	pool_debug("reap_handler: normally exited");
-	POOL_SETMASK(&UnBlockSig);
 }
 
 /*
@@ -1394,3 +1448,57 @@ pool_get_system_db_info(void)
 	
 	return system_db_info->info;
 }
+
+/*
+ * pool_pause: A process pauses by select().
+ */
+static int pool_pause(struct timeval *timeout)
+{
+	fd_set rfds;
+	int n;
+	char dummy;
+
+	FD_ZERO(&rfds);
+	FD_SET(pipe_fds[0], &rfds);
+	n = select(pipe_fds[0]+1, &rfds, NULL, NULL, timeout);
+	if (n == 1)
+		read(pipe_fds[0], &dummy, 1);
+	return n;
+}
+
+/*
+ * pool_pause: A process sleep using pool_pause().
+ *             If a signal event occurs, it raises signal handler.
+ */
+static void pool_sleep(unsigned int second)
+{
+	struct timeval current_time, sleep_time;
+
+	gettimeofday(&current_time, NULL);
+	sleep_time.tv_sec = second + current_time.tv_sec;
+	sleep_time.tv_usec = current_time.tv_usec;
+
+	POOL_SETMASK(&UnBlockSig);
+	while (sleep_time.tv_sec > current_time.tv_sec ||
+		   sleep_time.tv_usec > current_time.tv_usec)
+	{
+		struct timeval timeout;
+		int r;
+
+		timeout.tv_sec = sleep_time.tv_sec - current_time.tv_sec;
+		timeout.tv_usec = sleep_time.tv_usec - current_time.tv_usec;
+		if (timeout.tv_usec < 0)
+		{
+			timeout.tv_sec--;
+			timeout.tv_usec += 1000000;
+		}
+
+		r = pool_pause(&timeout);
+		POOL_SETMASK(&BlockSig);
+		if (r > 0)									
+			CHECK_REQUEST;
+		POOL_SETMASK(&UnBlockSig);
+		gettimeofday(&current_time, NULL);
+	}
+	POOL_SETMASK(&BlockSig);
+}
