@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.5 2007/05/17 06:46:02 yamaguti Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool.h,v 1.5.2.6 2007/09/14 11:06:35 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -64,11 +64,12 @@
 #define NO_STRICT_MODE(s) (strncasecmp((s), NO_STRICT_MODE_STR, strlen(NO_STRICT_MODE_STR)) == 0)
 
 typedef enum {
-    POOL_CONTINUE = 0,
+	POOL_CONTINUE = 0,
 	POOL_IDLE,
-    POOL_END,
-    POOL_ERROR,
-    POOL_FATAL
+	POOL_END,
+	POOL_ERROR,
+	POOL_FATAL,
+	POOL_DEADLOCK
 } POOL_STATUS;
 
 /* protocol major version numbers */
@@ -140,6 +141,7 @@ typedef struct {
 	int replication_stop_on_mismatch;		/* if there's a data mismatch between master and secondary
 											 * start degenration to stop replication mode
 											 */
+	int replicate_select; /* if non 0, replicate SELECT statement when load balancing is disabled. */
 	char **reset_query_list;		/* comma separated list of quries to be issued at the end of session */
 
 	int print_timestamp;		/* if non 0, print time stamp to each log line */
@@ -294,7 +296,7 @@ typedef struct {
 					   pool_config->backend_desc->num_backends))
 #endif
 #define NUM_BACKENDS (in_load_balance? (selected_slot+1) : \
-					  (((!REPLICATION && !PARALLEL_MODE) && !MASTER_SLAVE)? Req_info->master_node_id+1: \
+					  (((!REPLICATION && !PARALLEL_MODE) && !MASTER_SLAVE) ? Req_info->master_node_id+1: \
 					   pool_config->backend_desc->num_backends))
 #define BACKEND_INFO(backend_id) (pool_config->backend_desc->backend_info[(backend_id)])
 #define LOAD_BALANCE_STATUS(backend_id) (pool_config->load_balance_status[(backend_id)])
@@ -304,7 +306,7 @@ typedef struct {
 	 (BACKEND_INFO(backend_id).backend_status == CON_CONNECT_WAIT)))
 #define CONNECTION_SLOT(p, slot) ((p)->slots[(slot)])
 #define CONNECTION(p, slot) (CONNECTION_SLOT(p, slot)->con)
-#define MASTER_CONNECTION(p) ((p)->slots[Req_info->master_node_id])
+#define MASTER_CONNECTION(p) ((p)->slots[MASTER_NODE_ID])
 #define MASTER_NODE_ID (in_load_balance? selected_slot : Req_info->master_node_id)
 #define IS_MASTER_NODE_ID(node_id) (MASTER_NODE_ID == (node_id))
 //#define SECONDARY_CONNECTION(p) ((p)->slots[1])
@@ -361,6 +363,7 @@ typedef struct {
  */
 extern POOL_CONFIG *pool_config;	/* configuration values */
 extern POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
+extern volatile sig_atomic_t backend_timer_expired; /* flag for connection closed timer is expired */
 extern long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 extern int my_proc_id;  /* process table id (!= UNIX's PID) */
 extern POOL_SYSTEMDB_CONNECTION_POOL *system_db_info; /* systemdb */
@@ -380,6 +383,7 @@ extern void pool_log(const char *fmt,...);
 extern int pool_get_config(char *confpath);
 extern void do_child(int unix_fd, int inet_fd);
 extern void pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file);
+extern int select_load_balancing_node(void);
 extern int pool_init_cp(void);
 extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 									  POOL_CONNECTION_POOL *backend,
@@ -395,6 +399,7 @@ extern int pool_flush(POOL_CONNECTION *cp);
 extern int pool_flush_it(POOL_CONNECTION *cp);
 extern int pool_write_and_flush(POOL_CONNECTION *cp, void *buf, int len);
 extern char *pool_read_string(POOL_CONNECTION *cp, int *len, int line);
+extern int pool_unread(POOL_CONNECTION *cp, void *data, int len);
 
 extern int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 extern int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp);
@@ -403,6 +408,7 @@ extern int pool_init_cp(void);
 extern POOL_CONNECTION_POOL *pool_create_cp(void);
 extern POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor, int check_socket);
 extern void pool_discard_cp(char *user, char *database, int protoMajor);
+extern void pool_backend_timer(void);
 
 extern POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend, 
 								  POOL_CONNECTION_POOL *backend);
